﻿namespace DependencyPropertySerialization.Shared
{
    using System;
    using System.Reflection;
    using System.Windows;
    using System.ComponentModel;

    /// <summary>
    /// A helper class for DependencyProperties
    /// </summary>
    public class DependencyPropertyHelper
    {
        public static DependencyProperty FindDependencyProperty(object _object, string _propertyName)
        {
            return FindDependencyPropertyInternal(_object, _propertyName, true) as DependencyProperty;
        }

        public static DependencyProperty FindSerializableDependencyProperty(object _object, string _propertyName)
        {
            return FindDependencyPropertyInternal(_object, _propertyName, false) as DependencyProperty;
        }

        public static DependencyPropertyKey FindReadOnlyDependencyProperty(object _object, string _propertyName)
        {
            return FindDependencyPropertyInternal(_object, _propertyName, true) as DependencyPropertyKey;
        }

        public static bool IsSerializableDependencyProperty(DependencyObject _obj, PropertyDescriptor _descriptor)
        {
            if (null != _descriptor)
            {
                // Quick exit if possible
                if (_descriptor.IsReadOnly)
                {
                    return false;
                }

                DependencyProperty dp = DependencyPropertyHelper.FindSerializableDependencyProperty(_obj, _descriptor.Name);

                if (dp != null)
                {
                    return true;
                }

                return false;
                // Or the hacky way:
                /* 
                bool bValid = false;
                 * 
                foreach (Attribute attribRover in _descriptor.Attributes)
                {
                    if (attribRover.ToString() == "MS.Internal.ComponentModel.DependencyPropertyAttribute")
                    {
                        bValid = true;
                        break;
                    }
                }
                return bValid;
                */
            }
            else
            {
                throw new ArgumentNullException("_descriptor");
            }
        }

        /// <summary>
        /// Finds a dependency property on an object by its name.
        /// 
        /// Motivation: There is no accesible DP registry. When creating DPs, one must register them using
        /// DependencyProperty.Register(), but there is no DependencyProperty.FindProperty() - which might 
        /// be necessary in some cases, namely
        ///     - De-Serialization
        ///     - Duck typing
        /// </summary>
        /// <param name="_object">The object that supposedly has a property of the supplied name</param>
        /// <param name="_propertyName">The name of the property to find, e.g. "Height". 
        /// NOTE: The corresponding property is expected to be named "HeightProperty" in this case.</param>
        /// <param name="_acceptNotSerialized">If true, returns all DependencyProperties. If false, only those that are not marked as NonSerialized</param>
        /// <returns></returns>
        private static object FindDependencyPropertyInternal(object _object, string _propertyName, bool _acceptNotSerialized)
        {
            if (null != _object && !String.IsNullOrEmpty(_propertyName))
            {
                Type typeAttachedTo = _object.GetType();

                string propertyPropertyName = _propertyName + "Property";
                FieldInfo fi = null;
                while (null == fi && typeAttachedTo.BaseType != typeof(DependencyObject))
                {
                    fi = typeAttachedTo.GetField(propertyPropertyName);

                    if (null != fi && (fi.Attributes & FieldAttributes.NotSerialized) == FieldAttributes.NotSerialized)
                    {
                        // bail out: We found the property, but it's marked as NonSerialized, so we're done.
                        return null;
                    }

                    typeAttachedTo = typeAttachedTo.BaseType;
                }

                if (null != fi)
                {
                    return fi.GetValue(null);
                }
            }

            return null;
        }
    }
}
