﻿// Demonstration of DependencyProperty Serialization
//    Copyright (C) 2009 Christoph Menge
//
// This program is licensed under
// The Code Project Open License (CPOL) 1.02
//
namespace DependencyPropertySerialization
{
    using System;
    using System.IO;
    using System.IO.Compression;
    using System.Runtime.Serialization;
    using System.Runtime.Serialization.Formatters.Binary;
    using SimpleProfiler;

    /// <summary>
    /// Demonstrates serialization of WPF's DependencyProperties using reflection with the help of a 
    /// smart base class.
    /// 
    /// This implementation requires all derived classes to implement a serialization constructor.
    /// </summary>
    internal class Program
    {
        private static int iNodeCount = 0;

        private static void Main(string[] args)
        {
            bool printTrees = true;

            // About 30% time overhead on my machine with this test data
            bool compressData = true;

            // Creates a tree with variable width and height.
            // n = \sum_{i = 0}^{d} w^{i}
            // Use width = 2 for a binary tree.
            GraphNode rootNode = CreateTree(2, 2);

            if (printTrees)
            {
                rootNode.PrintTree();
            }

            Profiler profiler = new Profiler();

            // Serialize our data
            Console.WriteLine("Tree Created. Total Nodes: {0}", iNodeCount);
            Console.WriteLine("Serializing Tree ...");
            profiler.StartMeasure();
            SerializeGraph(rootNode, compressData);
            profiler.StopMeasure();
            Console.WriteLine("Serialization time: {0:F3}", profiler.GetLastTime());

            // De-Serialize it again
            Console.WriteLine("De-Serializing Tree ...");
            profiler.StartMeasure();
            GraphNode loadedGraph = DeserializeGraph(compressData);

            if (null == loadedGraph)
            {
                Console.WriteLine("Error, aborting...");
                Console.ReadKey();
                return;
            }

            profiler.StopMeasure();

            if (printTrees)
            {
                loadedGraph.PrintTree();
            }

            Console.WriteLine("De-Serialized Graph. Deserialization time: {0:F3}", profiler.GetLastTime());
            Console.WriteLine("Press any key to close.");
            Console.ReadKey();
        }

        private static void SerializeGraph(GraphNodeBase _rootNode, bool _compress)
        {
            FileStream fs = null;
            GZipStream gz = null;

            StreamingContext sc = new StreamingContext(StreamingContextStates.All);
            BinaryFormatter bf = new BinaryFormatter();
            
            try
            {
                fs = new FileStream("serialize.tmp", FileMode.Create);

                if (_compress)
                {
                     gz = new GZipStream(fs, CompressionMode.Compress);
                    bf.Serialize(gz, _rootNode);
                }
                else
                {
                    bf.Serialize(fs, _rootNode);
                }   
            }
            catch (SerializationException ex)
            {
                Console.WriteLine("An Error Occurred: {0}", ex.Message);
            }
            finally
            {
                if (null != gz)
                {
                    gz.Close();
                }

                if (null != fs)
                {
                    fs.Close();
                }
            }
        }

        private static GraphNode DeserializeGraph(bool _compressed)
        {
            FileStream fs = new FileStream("serialize.tmp", FileMode.Open);
            GZipStream gz = null;

            GraphNode loadedGraph = null;

            try
            {
                BinaryFormatter formatter = new BinaryFormatter();

                if (_compressed)
                {
                    gz = new GZipStream(fs, CompressionMode.Decompress);
                    loadedGraph = (GraphNode)formatter.Deserialize(gz);
                    if (null != gz)
                    {
                        gz.Close();
                    }
                }
                else
                {
                    loadedGraph = (GraphNode)formatter.Deserialize(fs);
                }
            }
            catch (SerializationException ex)
            {
                Console.WriteLine("Failed to deserialize. Reason: " + ex.Message);
            }
            finally
            {
                if (null != gz)
                {
                    gz.Close();
                }

                if (null != fs)
                {
                    fs.Close();
                }
            }

            return loadedGraph;
        }

        private static void CreateTreeRecursive(GraphNode _parent, int _width, int _depth, int _maxdepth)
        {
            if (_depth == _maxdepth)
            {
                return;
            }

            for (int i = 0; i < _width; ++i)
            {
                GraphNode node = new GraphNode(_parent);
                ++iNodeCount;
                // node.OnNameChanged += new GraphNode.OnNameChangedHandler(NodeNameChangedHandler);
                node.Name = String.Format("{0}.{1}", _parent.Name, i);

                // Apply some value so we can later check serialization worked out
                node.AutoProperty = String.Format("{0}", i << 3);

                CreateTreeRecursive(node, _width, _depth + 1, _maxdepth);
            }
        }

        private static GraphNode CreateTree(int _maxdepth, int _width)
        {
            GraphNode rootNode = new GraphNode(null);
            rootNode.Name = "Root";
            iNodeCount = 1;

            CreateTreeRecursive(rootNode, _width, 0, _maxdepth);

            return rootNode;
        }
    }
}
