﻿// Demonstration of DependencyProperty Serialization
//    Copyright (C) 2009 Christoph Menge
//
// This program is licensed under
// The Code Project Open License (CPOL) 1.02
//
namespace WPFDependencyPropertySerializationError
{
    using System;
    using System.IO;
    using System.IO.Compression;
    using System.Runtime.Serialization;
    using System.Runtime.Serialization.Formatters.Binary;
    using SimpleProfiler;


    /// <summary>
    /// This sample applications demonstrates the InvalidOperationException
    /// "Current local value enumeration is outdated because one or more local values have been set since its creation."
    /// 
    /// The idea here is to make DependencyProperties serializable through the back door, namely using
    /// a serialization surrogate. However, that does not work and instead produces this peculiar Exception.
    /// </summary>
    class Program
    {
        static void SerializeGraph(GraphNodeBase _rootNode, bool _compress)
        {
            FileStream fs = null;
            GZipStream gz = null;

            ISurrogateSelector selector = new MySurrogateSelector();
            StreamingContext sc = new StreamingContext(StreamingContextStates.All);
            BinaryFormatter bf = new BinaryFormatter();

            bf.SurrogateSelector = selector;

            try
            {
                fs = new FileStream("serialize.tmp", FileMode.Create);

                if (_compress)
                {
                    gz = new GZipStream(fs, CompressionMode.Compress);
                    bf.Serialize(gz, _rootNode);
                }
                else
                {
                    bf.Serialize(fs, _rootNode);
                }
            }
            catch (SerializationException ex)
            {
                Console.WriteLine("An Error Occurred: {0}", ex.Message);
            }
            finally
            {
                if (null != gz)
                {
                    gz.Close();
                }

                if (null != fs)
                {
                    fs.Close();
                }
            }
        }

        static GraphNode DeserializeGraph(bool _compressed)
        {
            FileStream fs = new FileStream("serialize.tmp", FileMode.Open);
            GZipStream gz = null;

            GraphNode loadedGraph = null;

            try
            {
                BinaryFormatter formatter = new BinaryFormatter();
                formatter.SurrogateSelector = new MySurrogateSelector();

                if (_compressed)
                {
                    gz = new GZipStream(fs, CompressionMode.Decompress);
                    loadedGraph = (GraphNode)formatter.Deserialize(gz);
                    if (null != gz)
                        gz.Close();
                }
                else
                {
                    loadedGraph = (GraphNode)formatter.Deserialize(fs);
                }
            }
            catch (SerializationException ex)
            {
                Console.WriteLine("Failed to deserialize. Reason: " + ex.Message);
            }
            finally
            {
                if (null != gz)
                {
                    gz.Close();
                }

                if (null != fs)
                {
                    fs.Close();
                }
            }

            return loadedGraph;
        }

        static int iNodeCount = 0;

        static void CreateTreeRecursive(GraphNode _parent, int _depth, int _maxdepth)
        {
            if (_depth == _maxdepth)
                return;

            for (int i = 0; i < 2; ++i)
            {
                GraphNode node = new GraphNode(_parent);
                ++iNodeCount;
                node.Name = String.Format("{0}.{1}", _parent.Name, i);

                CreateTreeRecursive(node, _depth + 1, _maxdepth);
            }
        }

        static GraphNode CreateTree(int _maxdepth)
        {
            GraphNode rootNode = new GraphNode(null);
            rootNode.Name = "Root";
            iNodeCount = 1;

            CreateTreeRecursive(rootNode, 0, _maxdepth);

            return rootNode;
        }

        static void Main(string[] args)
        {
            bool printTrees = true;

            // About 30% time overhead on my machine with this test data
            bool compressData = true;

            // Creates a full binary tree with (2^(n+1) - 1) nodes.
            GraphNode rootNode = CreateTree(2);

            if (printTrees)
            {
                rootNode.PrintTree();
            }

            Profiler profiler = new Profiler();

            Console.WriteLine("Tree Created. Total Nodes: {0}", iNodeCount);

            Console.WriteLine("Serializing Tree ...");
            profiler.StartMeasure();
            SerializeGraph(rootNode, compressData);
            profiler.StopMeasure();
            Console.WriteLine("Serialization time: {0:F3}", profiler.GetLastTime());

            Console.WriteLine("De-Serializing Tree ...");

            profiler.StartMeasure();
            GraphNode loadedGraph = DeserializeGraph(compressData);

            if (null == loadedGraph)
            {
                Console.WriteLine("Error, aborting...");
                Console.ReadKey();
                return;
            }

            loadedGraph.Name = "Test";
            profiler.StopMeasure();

            if (printTrees)
            {
                loadedGraph.PrintTree();
            }

            Console.WriteLine("De-Serialized Graph. Deserialization time: {0:F3}", profiler.GetLastTime());
            Console.WriteLine("Press any key to close.");
            Console.ReadKey();
        }
    }
}
